
# TO DO: 1 file maken, ook met de nativelikeness ratings

setwd('C:/Users/Martijn/Desktop/Accenten-breakpoint/')
load('spk.rda')
spk = spk[,c("LDavgUSPMI115","Speaker","BirthNation","Language","IsIELanguage","Age","Sex","AoEO","LR","OtherLangCnt","NonIELangCnt","IELangCnt","EngLearnMethod","PopSize.log","GNI.log","Schooling")]
colnames(spk) = c("LD","Speaker","Country","Language","IsIESpeaker","Age","Gender","AEO","LR","NrLang","NrLangNIE","NrLangIE","LearningStyle","PopSize.log","GNI.log","CountryEduYrs")

spk$LD.log = log(spk$LD)
spk$IsMale = (spk$Gender == 'male')*1
spk$Gender = NULL
spk$LD = NULL
spk$IsNaturalLearner = (spk$LearningStyle=='naturalistic')*1
spk$LearningStyle = NULL
spk = spk[,c("LD.log","Speaker","Country","Language","IsIESpeaker","Age","IsMale","AEO","LR","NrLang","NrLangNIE","NrLangIE","IsNaturalLearner","PopSize.log","GNI.log","CountryEduYrs")]
eng = spk
eng = droplevels(eng)

co = read.table('correl.txt',header=T)
co$LD = NULL
colnames(co) = ("Nativelikeness","Speaker")
accents = merge(co,eng,by="Speaker",all.y=T)	# human ratings
save(accents,file='accents.rda')
######


load('accents.rda')
eng = accents
library(mgcv)

# simple test to see if heteroscedasticity might be a problem
library(car)
m = lm(LD.log ~ AEO + Country, data=eng);
ncvTest(m) # no heteroscedasticity 


m0 = gam(LD.log ~ AEO + s(Country,bs='re') + s(Language,bs='re'), data=eng)
summary(m0) # random-effect factor 'Language' is not significant


# select breakpoint (duration 2.5 minutes)
deviances = rep(Inf, 25)
for (i in 1:25) { 
	cat(i)
	flush.console()
	breakpoint = i
	eng$ShiftedAEO = eng$AEO - breakpoint;
	eng$PastBreakPoint = as.factor(eng$ShiftedAEO > 0);
	m = gam(LD.log ~ ShiftedAEO:PastBreakPoint + s(Country,bs='re'), data=eng, method='ML')
	deviances[i] = deviance(m)
}

breakpoint = which(deviances == min(deviances)) 
breakpoint

eng$ShiftedAEO = eng$AEO - breakpoint
eng$PastBreakPoint = as.factor(eng$ShiftedAEO > 0)
m0 = gam(LD.log ~ AEO + s(Country,bs='re'), data=eng,method='ML')
m1 = gam(LD.log ~ ShiftedAEO:PastBreakPoint + s(Country,bs='re'), data=eng, method='ML')

# the model with breakpoint is better
AIC(m0) - AIC(m1)
summary(m1)


# visualization of breakpoint
my = lm(LD.log ~ ShiftedAEO:PastBreakPoint,data=eng)
mydf = as.data.frame(eng$AEO)
mydf$simple = predict(lm(LD.log ~ AEO, data=eng))
mydf$fit = predict(my,interval="confidence")[,1]
mydf$low = predict(my,interval="confidence")[,2]
mydf$high = predict(my,interval="confidence")[,3]
colnames(mydf)[1] = "AEO"
mydf = mydf[order(mydf$AEO),]

#pdf('Figure1.pdf',width=6.5,height=6.5,useDingbats=F)
plot(eng$AEO,eng$LD.log,col="gray",xlab="Age of English onset",ylab="Levenshtein distance (log-transformed) from American English",axes=F) 
ticks=c(0,breakpoint,10,20,30,40,50,60)
axis(side = 1, at = ticks)
axis(side = 2)
abline(v=breakpoint,col="darkgray")
lines(mydf$AEO,mydf$fit,lty=1)
lines(mydf$AEO,mydf$simple,lty=2)
polygon(c(mydf$AEO,rev(mydf$AEO)),c(mydf$high,rev(mydf$low)),col=rgb(.5,.5,.5,alpha=.25),border=NA)
box()
#dev.off()


# test stability of breakpoint
set.seed(666)
breakpoints = rep(0, 100)
#spkIEcnt = rep(0,100)
for (j in 1:100) { 
	# get new same-sized dataset by sampling with replacement
	dat = eng[c(sample(nrow(eng), nrow(eng), replace=T)), ] 
	cat("\n")
	flush.console()

	deviances = rep(Inf, 25)
	for (i in 1:25) { 
		cat(i)
		flush.console()
		breakpoint = i
		dat$ShiftedAEO = dat$AEO - breakpoint
		dat$PastBreakPoint = as.factor(dat$ShiftedAEO > 0)
		m = gam(LD.log ~ ShiftedAEO:PastBreakPoint + s(Country,bs='re'),data=dat, method='ML')
		deviances[i] = deviance(m)
	}
	breakpoints[j] = which(deviances == min(deviances))
	#spkIEcnt[j] = sum(dat$IsIESpeaker==TRUE)
}
table(breakpoints)
# 2  3  4  5  6  7  8 15 16 17 18 19 20 25 
# 3  3  2  2 43  4  1  1 20  4  5  5  6  1


# saved as the computation takes 30 minutes
save(breakpoints,file='breakpoints.rda') 
#save(spkIEcnt,file='spkIEcnt.rda')


# this difference might be caused by language background: split IE and non-IE
deviances = rep(Inf, 25^2)
dim(deviances) = c(25,25)
for (i in 1:25) {
	cat(i)
	flush.console()
	for (j in 1:25) { 
		breakpointIE = i
		breakpointNIE = j
		eng$ShiftedAEO = NA
		eng[eng$IsIESpeaker==T,]$ShiftedAEO = eng[eng$IsIESpeaker==T,]$AEO - breakpointIE
		eng[eng$IsIESpeaker==F,]$ShiftedAEO = eng[eng$IsIESpeaker==F,]$AEO - breakpointNIE
		eng$PastBreakPoint = as.factor(eng$ShiftedAEO > 0);

		m = gam(LD.log ~ ShiftedAEO:IsIESpeaker:PastBreakPoint + s(Country,bs='re'),data=eng, method='ML')
		deviances[i,j] = deviance(m)
	}
}

breakpointIE = which(deviances == min(deviances),arr.ind=T)[1] 
breakpointNIE = which(deviances == min(deviances),arr.ind=T)[2]
breakpointIE
breakpointNIE


# validation of two breakpoints:
set.seed(666)
breakpointsIE = rep(0, 100)
breakpointsNIE = rep(0, 100)
#spkIEcnt = rep(0,100)
for (k in 1:100) { 
	# get new same-sized dataset by sampling with replacement
	dat = eng[c(sample(nrow(eng), nrow(eng), replace=T)), ] 
	cat(paste(k,': ',sep=''))
	flush.console()

	deviances = rep(Inf, 25^2)
	dim(deviances) = c(25,25)
	for (i in min(dat$AEO):25) {
		cat(i)
		flush.console()
		for (j in min(dat$AEO):25) { 
			breakpointIE = i
			breakpointNIE = j
			dat$ShiftedAEO = NA
			dat[dat$IsIESpeaker==T,]$ShiftedAEO = dat[dat$IsIESpeaker==T,]$AEO - breakpointIE
			dat[dat$IsIESpeaker==F,]$ShiftedAEO = dat[dat$IsIESpeaker==F,]$AEO - breakpointNIE
			dat$PastBreakPoint = as.factor(dat$ShiftedAEO > 0)

			m = gam(LD.log ~ ShiftedAEO:IsIESpeaker:PastBreakPoint + s(Country,bs='re'),data=dat, method='ML')
			deviances[i,j] = deviance(m)
		}
	}

	breakpointsIE[k] = which(deviances == min(deviances),arr.ind=T)[1] 
	breakpointsNIE[k] = which(deviances == min(deviances),arr.ind=T)[2]

	cat(paste(' -',breakpointsNIE[k],breakpointsIE[k],'\n',sep=' '))
	flush.console()
}
table(breakpointsIE)
table(breakpointsNIE)
save(breakpointsIE,file='breakpointsIE.rda')
save(breakpointsNIE,file='breakpointsNIE.rda')


# the model with two breakpoints is better than a model with only one breakpoint
eng$ShiftedAEO = NA
eng[eng$IsIESpeaker==T,]$ShiftedAEO = eng[eng$IsIESpeaker==T,]$AEO - breakpointIE
eng[eng$IsIESpeaker==F,]$ShiftedAEO = eng[eng$IsIESpeaker==F,]$AEO - breakpointNIE
eng$PastBreakPoint = as.factor(eng$ShiftedAEO > 0)
m2 = gam(LD.log ~ ShiftedAEO:IsIESpeaker:PastBreakPoint + s(Country,bs='re'), data=eng, method='ML')
AIC(m1) - AIC(m2)
summary(m2)

# also the individual breakpoints are needed:
mNIE0 = gam(LD.log ~ AEO + s(Country,bs='re'), data=eng[eng$IsIESpeaker==F,], method='ML')
mNIE1 = gam(LD.log ~ ShiftedAEO:PastBreakPoint + s(Country,bs='re'), data=eng[eng$IsIESpeaker==F,], method='ML')
AIC(mNIE0) - AIC(mNIE1)

mIE0 = gam(LD.log ~ AEO + s(Country,bs='re'), data=eng[eng$IsIESpeaker==T,], method='ML')
mIE1 = gam(LD.log ~ ShiftedAEO:PastBreakPoint + s(Country,bs='re'), data=eng[eng$IsIESpeaker==T,], method='ML')
AIC(mIE0) - AIC(mIE1)



# visualization of breakpoints
engNIE = eng[eng$IsIESpeaker == F,]
my = lm(LD.log ~ ShiftedAEO:PastBreakPoint,data=engNIE)
mydf = as.data.frame(engNIE$AEO)
mydf$simple = predict(lm(LD.log ~ AEO, data=engNIE))
mydf$fit = predict(my,interval="confidence")[,1]
mydf$low = predict(my,interval="confidence")[,2]
mydf$high = predict(my,interval="confidence")[,3]
colnames(mydf)[1] = "AEO"
mydf = mydf[order(mydf$AEO),]

#pdf('Figure2.pdf',width=14,height=6.5,useDingbats=F)
par(mfrow=c(1,2))
plot(engNIE$AEO,engNIE$LD.log,col="gray",main="Non-Indo-European speakers",xlab="Age of English onset",ylab="Levenshtein distance (log-transformed) from American English",axes=F, ylim=c(-5.5,-4.1)) 
ticks=c(0,breakpointNIE,10,20,30,40,50,60)
axis(side = 1, at = ticks)
axis(side = 2)
abline(v=breakpointNIE,col="darkgray")
abline(h=mean(engNIE$LD.log),lty=3,col='gray')
lines(mydf$AEO,mydf$fit,lty=1)
lines(mydf$AEO,mydf$simple,lty=2)
polygon(c(mydf$AEO,rev(mydf$AEO)),c(mydf$high,rev(mydf$low)),col=rgb(.5,.5,.5,alpha=.25),border=NA)
box()

engIE = eng[eng$IsIESpeaker == T,]
my = lm(LD.log ~ ShiftedAEO:PastBreakPoint,data=engIE)
mydf = as.data.frame(engIE$AEO)
mydf$simple = predict(lm(LD.log ~ AEO, data=engIE))
mydf$fit = predict(my,interval="confidence")[,1]
mydf$low = predict(my,interval="confidence")[,2]
mydf$high = predict(my,interval="confidence")[,3]
colnames(mydf)[1] = "AEO"
mydf = mydf[order(mydf$AEO),]

plot(engIE$AEO,engIE$LD.log,col="gray",main="Indo-European speakers",xlab="Age of English onset",ylab="Levenshtein distance (log-transformed) from American English",axes=F, ylim=c(-5.5,-4.1)) 
ticks=c(0,10,breakpointIE,20,30,40,50,60)
axis(side = 1, at = ticks)
axis(side = 2)
abline(v=breakpointIE,col="darkgray")
abline(h=mean(engIE$LD.log),lty=3,col='gray')
lines(mydf$AEO,mydf$fit,lty=1)
lines(mydf$AEO,mydf$simple,lty=2)
polygon(c(mydf$AEO,rev(mydf$AEO)),c(mydf$high,rev(mydf$low)),col=rgb(.5,.5,.5,alpha=.25),border=NA)
box()
#dev.off()



############################ breakpoints not robust: so exclude breakpoints
# N.B. accents is equal to eng
# testing significance of the predictors for determining Levenshtein distance
model0 <- gam(LD.log ~ s(Country,bs='re'), data=accents, method='ML')
summary(model0)

model1 <- gam(LD.log ~ IsIESpeaker + s(Country,bs='re'), data=accents, method='ML')
summary(model1)$p.table
AIC(model0) - AIC(model1)

model2 <- gam(LD.log ~ IsIESpeaker + AEO + s(Country,bs='re'), data=accents, method='ML')
summary(model2)$p.table
AIC(model1) - AIC(model2)

model3 <- gam(LD.log ~ IsIESpeaker + AEO + LR + s(Country,bs='re'), data=accents, method='ML')
summary(model3)$p.table
AIC(model2) - AIC(model3)

model4 <- gam(LD.log ~ IsIESpeaker + AEO + LR + Age + s(Country,bs='re'), data=accents, method='ML')
summary(model4)$p.table
AIC(model3) - AIC(model4)

model5 <- gam(LD.log ~ IsIESpeaker + AEO + LR + Age + IsNaturalLearner + s(Country,bs='re'), data=accents, method='ML')
summary(model5)$p.table
AIC(model4) - AIC(model5)

model6 <- gam(LD.log ~ IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + s(Country,bs='re'), data=accents, method='ML')
summary(model6)$p.table
AIC(model5) - AIC(model6)

model7 <- gam(LD.log ~ IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model7)$p.table
AIC(model6) - AIC(model7)

# IsMale, NrLangIE, NrLangNIE, PopSize.log, GNI.log are not significant, nor improve the model
model7b <- gam(LD.log ~ IsMale + IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model7b)$p.table
AIC(model7) - AIC(model7b)

model7b <- gam(LD.log ~ NrLangIE + IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model7b)$p.table
AIC(model7) - AIC(model7b)

model7b <- gam(LD.log ~ NrLangNIE + IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model7b)$p.table
AIC(model7) - AIC(model7b)

model7b <- gam(LD.log ~ PopSize.log + IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model7b)$p.table
AIC(model7) - AIC(model7b)

model7b <- gam(LD.log ~ GNI.log + IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model7b)$p.table
AIC(model7) - AIC(model7b)


# significant interactions which decrease the AIC by at least 2 (NrLang:IsIESpeaker does not decrease AIC by 2)

# center LR and Age to facilitate interpretation of interactions
accents$LR.c = accents$LR - mean(accents$LR)
accents$Age.c = accents$Age - mean(accents$Age)

model8 <- gam(LD.log ~ IsIESpeaker + AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model8)$p.table
AIC(model7) - AIC(model8)

# visualize interaction (non-centered)
model8b <- gam(LD.log ~ IsIESpeaker + AEO + LR*Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
vis.gam(model8b,view=c("LR","Age"),plot.type='contour',color='gray',too.far=0.04,main='',xlab='Length of residence',ylab='Age')

model9 <- gam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model9)$p.table
AIC(model8) - AIC(model9)

# other interactions are not significant, or do not improve the model
model9b <- gam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + IsIESpeaker:NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model9b)$p.table
AIC(model9) - AIC(model9b)


# testing non-linearities, ML seems to be more conservatieve than REML, so we'll stick with ML to prevent overfitting
model9b <- gam(LD.log ~ IsIESpeaker + s(AEO,by=IsIESpeaker) + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
AIC(model9) - AIC(model9b)

model9b <- gam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + te(LR.c,Age.c) + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
AIC(model9) - AIC(model9b)

model9b <- gam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + s(CountryEduYrs) + NrLang + s(Country,bs='re'), data=accents, method='ML')
AIC(model9) - AIC(model9b)

model9b <- gam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + s(NrLang,k=4) + s(Country,bs='re'), data=accents, method='ML')
AIC(model9) - AIC(model9b)

# testing random slopes, bam is used as it is faster
model9 <- bam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='REML')
model10 <- bam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,IsNaturalLearner,bs='re'), data=accents, method='REML')
summary(model10)$s.table
AIC(model9) - AIC(model10)

# other random slopes are not significant
model9c <- bam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,IsIESpeaker,bs='re'), data=accents, method='REML')
summary(model9c)$s.table

model9c <- bam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,AEO,bs='re'), data=accents, method='REML')
summary(model9c)$s.table

accents$IsIESpeaker_AEO = accents$AEO
accents[accents$IsIESpeaker==F,]$IsIESpeaker_AEO = 0
model9c <- bam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,IsIESpeaker_AEO,bs='re'), data=accents, method='REML')
summary(model9c)$s.table

model9c <- bam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,LR,bs='re'), data=accents, method='REML')
summary(model9c)$s.table

model9c <- bam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,Age,bs='re'), data=accents, method='REML')
summary(model9c)$s.table

accents$Age_LR = accents$Age * accents$LR
model9c <- bam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,Age_LR,bs='re'), data=accents, method='REML')
summary(model9c)$s.table

model9c <- bam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,CountryEduYrs,bs='re'), data=accents, method='REML')
summary(model9c)$s.table

model9c <- bam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,NrLang,bs='re'), data=accents, method='REML')
summary(model9c)$s.table


# final model: 
model10 <- gam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,IsNaturalLearner,bs='re'), data=accents, method='REML')
summary(model10)

# Model criticism
library(car)
qqp(resid(model10),col.lines='darkgray') # approximately normally distributed, 1 outlier
devfits = abs(resid(model10,"deviance"))
plot(devfits,type="h") # 1 outlier
accents2 = accents[-c(which(devfits==max(devfits))),]
finalmodel <- gam(LD.log ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,IsNaturalLearner,bs='re'), data=accents2, method='REML')
qqp(resid(finalmodel),col.lines='darkgray')
summary(finalmodel)


# determine effect size of interaction by calculating the range of the total effect
mx = -Inf
mn = Inf
accents2$LR.c_Age.c = accents2$LR.c * accents2$Age.c
smry = summary(finalmodel)$p.table
for (i in 1:nrow(accents2)) {
    val = smry["LR.c","Estimate"]*accents2[i,"LR.c"] + 
          smry["Age.c","Estimate"]*accents2[i,"Age.c"] + 
          smry["LR.c:Age.c","Estimate"]*accents2[i,"LR.c_Age.c"]
    if (val < mn) { 
        mn = val
    }
    if (val > mx) { 
        mx = val
    }
}
mx - mn

# effect size of AEO (separate for IE vs. non-IE speakers
accentsNIE = accents2[accents2$IsIESpeaker==F,]
round((max(accentsNIE$AEO) - min(accentsNIE$AEO))*summary(finalmodel)$p.table["IsIESpeakerFALSE:AEO","Estimate"],3)

accentsIE = accents2[accents2$IsIESpeaker==T,]
round((max(accentsIE$AEO) - min(accentsIE$AEO))*summary(finalmodel)$p.table["IsIESpeakerTRUE:AEO","Estimate"],3)

# effect size of remaining variables
round((max(accents2$CountryEduYrs) - min(accents2$CountryEduYrs))*summary(finalmodel)$p.table["CountryEduYrs","Estimate"],3)
round((max(accents2$NrLang) - min(accents2$NrLang))*summary(finalmodel)$p.table["NrLang","Estimate"],3)
round(summary(finalmodel)$p.table["IsIESpeakerTRUE","Estimate"],3)
round(summary(finalmodel)$p.table["IsNaturalLearner","Estimate"],3)
round(summary(finalmodel)$p.table["IsIESpeakerTRUE","Estimate"],3)


######
# test with the other dependent variable
accents = accents[!is.na(accents$Nativelikeness),]
accents = droplevels(accents)
hr = accents


# test for possible heteroscedasticity
library(car)
m = lm(Nativelikeness ~ AEO + Country, data=hr);
ncvTest(m) # severe heteroscedasticity 

# log-transformation to correct for heteroscedasticity
m = lm(log(Nativelikeness) ~ AEO + Country, data=hr);
ncvTest(m)

m0 = gam(log(Nativelikeness) ~ AEO + s(Country,bs='re') + s(Language,bs='re'), data=hr)
summary(m0) # random-effect factor 'Language' is not significant


# select breakpoint (duration 10 seconds), also 6
deviances = rep(Inf, 25)
for (i in 1:25) { 
	cat(i)
	flush.console()
	breakpoint = i
	hr$ShiftedAEO = hr$AEO - breakpoint;
	hr$PastBreakPoint = as.factor(hr$ShiftedAEO > 0);
	m = gam(log(Nativelikeness) ~ ShiftedAEO:PastBreakPoint + s(Country,bs='re'), data=hr, method='ML')
	deviances[i] = deviance(m)
}

breakpoint = which(deviances == min(deviances)) 
breakpoint

hr$ShiftedAEO = hr$AEO - breakpoint
hr$PastBreakPoint = as.factor(hr$ShiftedAEO > 0)
m0 = gam(log(Nativelikeness) ~ AEO + s(Country,bs='re'), data=hr,method='ML')
m1 = gam(log(Nativelikeness) ~ ShiftedAEO:PastBreakPoint + s(Country,bs='re'), data=hr, method='ML')

# the model with breakpoint is better
AIC(m0) - AIC(m1)
summary(m1)




# test stability of breakpoint
set.seed(666)
breakpointsNat = rep(0, 100)
for (j in 1:100) { 
	# get new same-sized dataset by sampling with replacement
	dat = hr[c(sample(nrow(hr), nrow(hr), replace=T)), ] 
	cat("\n")
	flush.console()

	deviances = rep(Inf, 25)
	for (i in min(dat$AEO):20) { # smaller set to reduce time 
		cat(i)
		flush.console()
		breakpoint = i
		dat$ShiftedAEO = dat$AEO - breakpoint
		dat$PastBreakPoint = as.factor(dat$ShiftedAEO > 0)
		m = gam(log(Nativelikeness) ~ ShiftedAEO:PastBreakPoint + s(Country,bs='re'),data=dat, method='ML')
		deviances[i] = deviance(m)
	}
	breakpointsNat[j] = which(deviances == min(deviances))
}
table(breakpointsNat)
#breakpointsHR
# 5  6  7  8  9 10 15 16 17 18 19 20 
#10 52  2  4  1  1 16  2  2  7  1  2 

# saved as the computation takes 15 minutes
save(breakpointsNat,file='breakpointsNat.rda') 


# this difference is caused by language background: split IE and non-IE

deviances = rep(Inf, 25^2)
dim(deviances) = c(25,25)
for (i in 1:25) {
	cat(i)
	flush.console()
	for (j in 1:25) { 
		breakpointIE = i
		breakpointNIE = j
		hr$ShiftedAEO = NA
		hr[hr$IsIESpeaker==T,]$ShiftedAEO = hr[hr$IsIESpeaker==T,]$AEO - breakpointIE
		hr[hr$IsIESpeaker==F,]$ShiftedAEO = hr[hr$IsIESpeaker==F,]$AEO - breakpointNIE
		hr$PastBreakPoint = as.factor(hr$ShiftedAEO > 0);

		m = gam(log(Nativelikeness) ~ ShiftedAEO:IsIESpeaker:PastBreakPoint + s(Country,bs='re'),data=hr, method='ML')
		deviances[i,j] = deviance(m)
	}
}

breakpointIE = which(deviances == min(deviances),arr.ind=T)[1] 
breakpointNIE = which(deviances == min(deviances),arr.ind=T)[2]
breakpointIE # also 16 
breakpointNIE # also 6

# the model with two breakpoints is better than a model with only one breakpoint
hr$ShiftedAEO = NA
hr[hr$IsIESpeaker==T,]$ShiftedAEO = hr[hr$IsIESpeaker==T,]$AEO - breakpointIE
hr[hr$IsIESpeaker==F,]$ShiftedAEO = hr[hr$IsIESpeaker==F,]$AEO - breakpointNIE
hr$PastBreakPoint = as.factor(hr$ShiftedAEO > 0);
m2 = gam(log(Nativelikeness) ~ ShiftedAEO:IsIESpeaker:PastBreakPoint + s(Country,bs='re'), data=hr, method='ML')
AIC(m1) - AIC(m2)
summary(m2)

# also the individual breakpoints are needed:
mNIE0 = gam(log(Nativelikeness) ~ AEO + s(Country,bs='re'), data=hr[hr$IsIESpeaker==F,], method='ML')
mNIE1 = gam(log(Nativelikeness) ~ ShiftedAEO:PastBreakPoint + s(Country,bs='re'), data=hr[hr$IsIESpeaker==F,], method='ML')
AIC(mNIE0) - AIC(mNIE1)

mIE0 = gam(log(Nativelikeness) ~ AEO + s(Country,bs='re'), data=hr[hr$IsIESpeaker==T,], method='ML')
mIE1 = gam(log(Nativelikeness) ~ ShiftedAEO:PastBreakPoint + s(Country,bs='re'), data=hr[hr$IsIESpeaker==T,], method='ML')
AIC(mIE0) - AIC(mIE1)




# validation of two breakpoints (Nativelikeness data)
set.seed(666)
breakpointsNatIE = rep(0, 100)
breakpointsNatNIE = rep(0, 100)
#spkIEcnt = rep(0,100)
for (k in 1:100) { 
	# get new same-sized dataset by sampling with replacement
	dat = hr[c(sample(nrow(hr), nrow(hr), replace=T)), ] 
	cat(paste(k,": ",sep=''))
	flush.console()

	deviances = rep(Inf, 25^2)
	dim(deviances) = c(25,25)
	for (i in min(dat$AEO):25) {
		cat(i)
		flush.console()
		for (j in min(dat$AEO):25) { 
			breakpointIE = i
			breakpointNIE = j
			dat$ShiftedAEO = NA
			dat[dat$IsIESpeaker==T,]$ShiftedAEO = dat[dat$IsIESpeaker==T,]$AEO - breakpointIE
			dat[dat$IsIESpeaker==F,]$ShiftedAEO = dat[dat$IsIESpeaker==F,]$AEO - breakpointNIE
			dat$PastBreakPoint = as.factor(dat$ShiftedAEO > 0);

			m = gam(log(Nativelikeness) ~ ShiftedAEO:IsIESpeaker:PastBreakPoint + s(Country,bs='re'),data=dat, method='ML')
			deviances[i,j] = deviance(m)
		}
	}

	breakpointsNatIE[k] = which(deviances == min(deviances),arr.ind=T)[1] 
	breakpointsNatNIE[k] = which(deviances == min(deviances),arr.ind=T)[2]
	cat(paste('-',breakpointsNatNIE[k],breakpointsNatIE[k],'\n',sep=' '))
	flush.console()
}
table(breakpointsNatIE)
table(breakpointsNatNIE)
save(breakpointsNatIE,file='breakpointsNatIE.rda')
save(breakpointsNatNIE,file='breakpointsNatNIE.rda')


############################ breakpoints not robust: so exclude breakpoints
# N.B. accents is equal to eng
# testing significance of the predictors for determining Levenshtein distance
accents = accents[!is.na(accents$Nativelikeness),]
accents = droplevels(accents)


model0 <- gam(log(Nativelikeness) ~ s(Country,bs='re'), data=accents, method='ML')
summary(model0)

model1 <- gam(log(Nativelikeness) ~ IsIESpeaker + s(Country,bs='re'), data=accents, method='ML')
summary(model1)$p.table
AIC(model0) - AIC(model1)

model2 <- gam(log(Nativelikeness) ~ IsIESpeaker + AEO + s(Country,bs='re'), data=accents, method='ML')
summary(model2)$p.table
AIC(model1) - AIC(model2)

model3 <- gam(log(Nativelikeness) ~ IsIESpeaker + AEO + LR + s(Country,bs='re'), data=accents, method='ML')
summary(model3)$p.table
AIC(model2) - AIC(model3)

model4 <- gam(log(Nativelikeness) ~ IsIESpeaker + AEO + LR + Age + s(Country,bs='re'), data=accents, method='ML')
summary(model4)$p.table
AIC(model3) - AIC(model4)

model5 <- gam(log(Nativelikeness) ~ IsIESpeaker + AEO + LR + Age + IsNaturalLearner + s(Country,bs='re'), data=accents, method='ML')
summary(model5)$p.table
AIC(model4) - AIC(model5)

model6 <- gam(log(Nativelikeness) ~ IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + s(Country,bs='re'), data=accents, method='ML')
summary(model6)$p.table
AIC(model5) - AIC(model6)

model7 <- gam(log(Nativelikeness) ~ IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model7)$p.table
AIC(model6) - AIC(model7)

# IsMale, NrLangIE, NrLangNIE, PopSize.log, GNI.log are not significant, nor improve the model
model7b <- gam(log(Nativelikeness) ~ IsMale + IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model7b)$p.table
AIC(model7) - AIC(model7b)

model7b <- gam(log(Nativelikeness) ~ NrLangIE + IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model7b)$p.table
AIC(model7) - AIC(model7b)

model7b <- gam(log(Nativelikeness) ~ NrLangNIE + IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model7b)$p.table
AIC(model7) - AIC(model7b)

model7b <- gam(log(Nativelikeness) ~ PopSize.log + IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model7b)$p.table
AIC(model7) - AIC(model7b)

model7b <- gam(log(Nativelikeness) ~ GNI.log + IsIESpeaker + AEO + LR + Age + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model7b)$p.table
AIC(model7) - AIC(model7b)


# significant interactions which decrease the AIC by at least 2 (NrLang:IsIESpeaker does not decrease AIC by 2)

# center LR and Age to facilitate interpretation of interactions
accents$LR.c = accents$LR - mean(accents$LR)
accents$Age.c = accents$Age - mean(accents$Age)

model8 <- gam(log(Nativelikeness) ~ IsIESpeaker + AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model8)$p.table
AIC(model7) - AIC(model8)

# this interaction is nog significant
model8b <- gam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model8b)$p.table
AIC(model8) - AIC(model8b)

# other interactions are not significant, or do not improve the model
model8b <- gam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + IsIESpeaker:NrLang + s(Country,bs='re'), data=accents, method='ML')
summary(model8b)$p.table
AIC(model8) - AIC(model8b)


# testing non-linearities, ML seems to be more conservative than REML, so we'll stick with ML to prevent overfitting
model8b <- gam(log(Nativelikeness) ~ IsIESpeaker + s(AEO,by=IsIESpeaker) + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
AIC(model8) - AIC(model8b)

model8b <- gam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + te(LR.c,Age.c) + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='ML')
AIC(model8) - AIC(model8b)

model8b <- gam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + s(CountryEduYrs) + NrLang + s(Country,bs='re'), data=accents, method='ML')
AIC(model8) - AIC(model8b)

model8b <- gam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + s(NrLang,k=4) + s(Country,bs='re'), data=accents, method='ML')
AIC(model8) - AIC(model8b)

# testing random slopes, bam is used as it is faster, all n.s.
model8 <- gam(log(Nativelikeness) ~ IsIESpeaker + AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='REML')
model8c <- gam(log(Nativelikeness) ~ IsIESpeaker + AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,IsNaturalLearner,bs='re'), data=accents, method='REML')
summary(model8c)$s.table

model8c <- bam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,IsIESpeaker,bs='re'), data=accents, method='REML')
summary(model8c)$s.table

model8c <- bam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,AEO,bs='re'), data=accents, method='REML')
summary(model8c)$s.table

accents$IsIESpeaker_AEO = accents$AEO
accents[accents$IsIESpeaker==F,]$IsIESpeaker_AEO = 0
model8c <- bam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,IsIESpeaker_AEO,bs='re'), data=accents, method='REML')
summary(model8c)$s.table

model8c <- bam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,LR,bs='re'), data=accents, method='REML')
summary(model8c)$s.table

model8c <- bam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,Age,bs='re'), data=accents, method='REML')
summary(model8c)$s.table

accents$Age_LR = accents$Age * accents$LR
model8c <- bam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,Age_LR,bs='re'), data=accents, method='REML')
summary(model8c)$s.table

model8c <- bam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,CountryEduYrs,bs='re'), data=accents, method='REML')
summary(model8c)$s.table

model8c <- bam(log(Nativelikeness) ~ IsIESpeaker + IsIESpeaker:AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re') + s(Country,NrLang,bs='re'), data=accents, method='REML')
summary(model8c)$s.table


# final model: 
model8 <- gam(log(Nativelikeness) ~ IsIESpeaker + AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang + s(Country,bs='re'), data=accents, method='REML')
summary(model8) 

# random intercept not significant
finalmodel <- gam(log(Nativelikeness) ~ IsIESpeaker + AEO + LR.c*Age.c + IsNaturalLearner + CountryEduYrs + NrLang, data=accents, method='REML')
summary(finalmodel)

# Model criticism
library(car)
qqp(resid(model9)) # approximately normally distributed, 1 outlier
devfits = abs(resid(model10,"deviance"))
plot(devfits,type="h") # no real outliers


# determine effect size of interaction by calculating the range of the total effect (note that this dependent variable has a different scale)
mx = -Inf
mn = Inf
accents2$LR.c_Age.c = accents2$LR.c * accents2$Age.c
smry = summary(finalmodel)$p.table
for (i in 1:nrow(accents2)) {
    val = smry["LR.c","Estimate"]*accents2[i,"LR.c"] + 
          smry["Age.c","Estimate"]*accents2[i,"Age.c"] + 
          smry["LR.c:Age.c","Estimate"]*accents2[i,"LR.c_Age.c"]
    if (val < mn) { 
        mn = val
    }
    if (val > mx) { 
        mx = val
    }
}
mx - mn

# effect size of remaining variables
round((max(accents$AEO) - min(accents$AEO))*summary(finalmodel)$p.table["AEO","Estimate"],3)
round((max(accents2$CountryEduYrs) - min(accents2$CountryEduYrs))*summary(finalmodel)$p.table["CountryEduYrs","Estimate"],3)
round((max(accents2$NrLang) - min(accents2$NrLang))*summary(finalmodel)$p.table["NrLang","Estimate"],3)
round(summary(finalmodel)$p.table["IsIESpeakerTRUE","Estimate"],3)
round(summary(finalmodel)$p.table["IsNaturalLearner","Estimate"],3)
round(summary(finalmodel)$p.table["IsIESpeakerTRUE","Estimate"],3)



